@@ -13,7 +13,7 @@ use Hook::LexWrap;
 use HTML::Display qw();
 
 use vars qw( $VERSION @EXPORT );
-$VERSION = '0.29';
+$VERSION = '0.30';
 @EXPORT = qw( &shell );
 
 =head1 NAME
@@ -108,9 +108,9 @@ sub init {
   # Install the request dumper :
   $self->{request_wrapper} = wrap *LWP::UserAgent::request,
                                pre => sub { $self->request_dumper($_[1]) if $self->option("dumprequests"); },
-                               post => sub { 
+                               post => sub {
                                  #warn scalar @_, " arguments";
-                                 #warn $_ for @_; 
+                                 #warn $_ for @_;
                                  $self->response_dumper($_[-1]) if $self->option("dumpresponses");
                                };
 
@@ -221,7 +221,7 @@ sub print_paged {
     print $fh $_ for @_;
     close $fh;
 
-    my @pagers = ($ENV{PAGER});
+    my @pagers = ($ENV{PAGER},qq{"$^X" -p});
 		foreach my $pager (@pagers) {
 			if ($^O eq 'VMS') {
 				last if system("$pager $filename") == 0; # quoting prevents logical expansion
@@ -304,6 +304,26 @@ sub re_or_string {
   $arg;
 };
 
+=head2 C<< $shell->link_text LINK >>
+
+Returns a meaningfull text from a WWW::Mechanize::Link object. This is (in order of
+precedence) :
+
+    $link->text
+    $link->name
+    $link->url
+
+=cut
+
+sub link_text {
+  my ($self,$link) = @_;
+  my $result;
+  for (qw( text name url )) {
+    $result = $link->$_ and last;
+  };
+  $result;
+};
+
 =head2 C<$shell-E<gt>history>
 
 Returns the (relevant) shell history, that is, all commands
@@ -340,7 +360,7 @@ use WWW::Mechanize;
 use WWW::Mechanize::FormFiller;
 use URI::URL;
 
-my $agent = WWW::Mechanize->new();
+my $agent = WWW::Mechanize->new( autocheck => 1 );
 my $formfiller = WWW::Mechanize::FormFiller->new();
 $agent->env_proxy();
 HEADER
@@ -534,12 +554,12 @@ sub run_save {
   if (ref $user_link) {
     my $count = -1;
     my $re = $user_link;
-    @links = map { $count++; (($_->[0] =~ /$re/)||($_->[1] =~ /$re/)) ? $count : () } @all_links;
+    @links = map { $count++; (($_->text =~ /$re/)||($_->url =~ /$re/)) ? $count : () } @all_links;
     if (@links == 0) {
       print "No match for /$re/.\n";
     };
     push @history, q{my $count = -1;} . "\n";
-    push @history, sprintf q{@links = map { $count++; (($_->[0] =~ qr(%s))||($_->[1] =~ qr(%s))) ? $count : () } @all_links;} . "\n", $re, $re;
+    push @history, sprintf q{@links = map { $count++; (($_->text =~ qr(%s))||($_->url =~ qr(%s))) ? $count : () } @all_links;} . "\n", $re, $re;
   } else {
     @links = $user_link;
     push @history, sprintf q{@links = '%s';} . "\n", $user_link;
@@ -549,7 +569,7 @@ sub run_save {
     $self->add_history( @history,<<'CODE' );
   my $base = $agent->uri;
   for my $link (@links) {
-    my $target = $all_links[$link]->[0];
+    my $target = $all_links[$link]->url;
     my $url = URI::URL->new($target,$base);
     $target = $url->path;
     $target =~ s!^(.*/)?([^/]+)$!$2!;
@@ -570,7 +590,7 @@ sub run_save {
 CODE
     my $base = $self->agent->uri;
     for my $link (@links) {
-      my $target = $all_links[$link]->[0];
+      my $target = $all_links[$link]->url;
       my $url = URI::URL->new($target,$base);
       $target = $url->path;
       $target =~ s!^(.*/)?([^/]+)$!$2!;
@@ -667,10 +687,11 @@ select a link to follow.
 
 sub run_links {
   my ($self) = @_;
-  my $links = $self->agent->links;
+  my @links = $self->agent->links;
   my $count = 0;
-  for my $link (@$links) {
-    print "[", $count++, "] ", $link->[1],"\n";
+  for my $link (@links) {
+    # print "[", $count++, "] ", $link->[1],"\n";
+    print sprintf "[%s] %s\n", $count++, $self->link_text($link);
   };
 };
 
@@ -942,8 +963,8 @@ sub run_open {
   if (ref $link) {
     my $re = $link;
     my $count = -1;
-    my @possible_links = @{$self->agent->links()};
-    my @links = map { $count++; $_->[1] =~ /$re/ ? $count : () } @possible_links;
+    my @possible_links = $self->agent->links();
+    my @links = map { $count++; $_->text =~ /$re/ ? $count : () } @possible_links;
     if (@links > 1) {
       $self->print_pairs([ @links ],[ map {$possible_links[$_]->[1]} @links ]);
       undef $link;
@@ -953,7 +974,7 @@ sub run_open {
     } else {
       $self->status( "Found $links[0]\n" );
       $link = $links[0];
-      if ($possible_links[$count]->[0] =~ /^javascript:(.*)/i) {
+      if ($possible_links[$count]->url =~ /^javascript:(.*)/i) {
         print "Can't follow javascript link $1\n";
         undef $link;
       };
@@ -977,8 +998,7 @@ sub run_open {
 # Complete partially typed links :
 sub comp_open {
   my ($self,$word,$line,$start) = @_;
-  # return grep {/^$word/} map {$_->[1]} (@{$self->agent->extract_links()});
-  my @completions = eval { grep {/^$word/} map {$_->[1]} (@{$self->agent->find_all_links()}) };
+  my @completions = eval { grep {/^$word/} map { $self->link_text( $_ )} ($self->agent->find_all_links()) };
   $self->display_user_warning($@) if $@;
   return @completions;
 };
@@ -1113,6 +1133,21 @@ sub run_script {
   $self->display($filename,$self->script("  "));
 };
 
+=head2 comment
+
+Adds a comment to the script and the history. The comment
+is prepended with a \n to increase readability.
+
+=cut
+
+sub run_comment {
+  my $self = shift;
+  if (@_)
+  {
+        $self->add_history("\n# @_ ");
+  }
+}
+
 =head2 fillout
 
 Fill out the current form
@@ -1510,6 +1545,7 @@ sub run_referrer {
     if ($@) {
       print "Could not set referrer : $@";
     };
+    # warn "Added $referrer";
     $self->add_history( sprintf q{$agent->add_header('Referer', '%s');}, $referrer);
   } else {
     # print "syntax: referer|referrer URL\n";
@@ -1519,7 +1555,16 @@ sub run_referrer {
   }
 };
 
-sub alias_referrer { qw(referer) };
+=head2 referer
+
+Alias for referrer
+
+=cut
+
+sub run_referer { 
+  goto &WWW::Mechanize::Shell::run_referrer 
+};
+# sub alias_referrer { qw(referer) };
 
 =head2 response
 
@@ -1702,6 +1747,14 @@ To fake an input field from within a shell session, use the C<eval> command :
 
 And yes, the generated script should do the Right Thing for this eval as well.
 
+=head1 LOCAL FILES
+
+If you want to use the shell on a local file without setting up a C<http> server
+to serve the file, you can use the C<file:> URI scheme to load it into the "browser":
+
+  get file:local.html
+  forms
+
 =head1 PROXY SUPPORT
 
 Currently, the proxy support is realized via a call to
@@ -1753,12 +1806,6 @@ The shell currently detects when you want to follow a JavaScript link and tells
 that this is not supported. It would be nicer if there was some callback mechanism
 to (automatically?) extract URLs from JavaScript-infected links.
 
-=item *
-
-The embedded test C<t/embedded-WWW-Mechanize-Shell.t> currently dies under Perl 5.8
-and Solaris after successfully running all tests. I can't test this myself so I don't
-know where the reason for that lies - any hints are welcome !
-
 =back
 
 =head1 TODO
